part of plot_window;

class HeatmapPlotWindow extends PlotWindow {

  Heatmap _heatmap;
  Heatmap get heatmap=>_heatmap;
  void set heatmap(Heatmap h) {
    this._heatmap=h;
    this._paddingRight = null;
  }

  void removeData() {
    heatmap=null;
  }
  
  HeatmapPlotWindow(CanvasElement canvas):super(canvas);

  Rectangle getInitialSize()=>new Rectangle(0, 0, heatmap.width, heatmap.height);

  num _paddingRight;
  num get paddingRight {
    if(_paddingRight==null) {
      context.save();
      context.font=getFontString(); 
      var getWidth = (String s)=>context.measureText(s).width;
      num maxCapWidth = getWidth(heatmap.maxCapAsString);
      num minCapWidth = getWidth(heatmap.minCapAsString);
      context.restore();
      _paddingRight=fontSize.value*4 +
        math.max(maxCapWidth, minCapWidth);
    }
    return _paddingRight;
  }
  
  void drawContent() {
    num xweight = heatmap.width/heatmap.values.length;
    num yweight = heatmap.height/heatmap.values[0].length;
    for (int i = 0; i<heatmap.values.length; i++) {
      for (int j = 0; j<heatmap.values[i].length; j++) {
        context.save();
        context.fillStyle = "${heatmap.toColor(heatmap.values[i][j])}";
        Point leftBottom = toCanvas(new Point(i*xweight, j*yweight));
        Point rightTop = toCanvas(new Point((i+1)*xweight, (j+1)*yweight));
        context.fillRect(leftBottom.x, leftBottom.y, rightTop.x - leftBottom.x, rightTop.y-leftBottom.y);
        context.restore();
      }
    }
    blankOutBorders();
    drawScale();
  }

  void drawScale() {
    context.save();
    context.strokeStyle="${Color.BLACK}";
    context.lineWidth=1;
    num xStart = canvas.width - paddingRight + fontSize.value;
    num height = canvas.height - paddingBottom-paddingTop;
    num width = fontSize.value*2;
    context.strokeRect(xStart, paddingTop, width, height);
    context.textAlign="start";
    context.textBaseline="middle";
    Point center = new Point(canvas.width - paddingRight + fontSize.value*4, paddingTop);

    context.fillText("${heatmap.maxCapAsString}", center.x, center.y);
    center = new Point(canvas.width - paddingRight + fontSize.value*4, canvas.height - paddingBottom);
    context.fillText("${heatmap.minCapAsString}", center.x, center.y);
    context.lineWidth=1.0;
    num yStart = canvas.height - paddingBottom;
    
    num factor = 1/height * (heatmap.maxCap - heatmap.minCap);
    var getFactor = (int i)=>factor*i + heatmap.minCap;
    for(int i=0; i<height; i++) {
      context.beginPath();
      var color = heatmap.toColor(i*factor);
      context.strokeStyle="${heatmap.toColor(getFactor(i))}"; //i*factor + heatmap.minValue)}";
      context.moveTo(xStart, yStart-i);
      context.lineTo(xStart+width, yStart-i);
      context.stroke();
    }
    context.restore();
  }
  
  Iterable<LegendEntry> getLegendEntries() {
    return [new LegendEntry(Color.BLACK, [heatmap.name], heatmap)];
  }
}