part of plot_window;

class LinePlotWindow extends PlotWindow {

  List<Line> lines = new List<Line>();

  LinePlotWindow(CanvasElement canvas):super(canvas);
  
  void addLine(Line line) {
    if (line.pointList == null || line.validPointList.points.isEmpty || lines.contains(line)) return;
    if (line.color==null) {
      line.color=colorIterator.getNext();
      print("color picked: ${line.color}");
    }
    lines.add(line);
  }
  
  void removeData() {
    this.lines.clear();
  }
  
  void removeLine(String s) {
    this.lines.remove(s);
  }

  Rectangle getInitialSize()=>lines.map((Line l)=>l.rectangle).reduce((value, element)=>value.boundingBox(element));

  void drawContent() {
    for (Line line in lines) {
      var points = line.smoothedPointList.points;
      points = filterToRectangle(points);
      if (points.isNotEmpty) {
        if (line.lineType.hasLines) {
          context.save();
          context.setStrokeColorRgb(line.color.red, line.color.green, line.color.blue);
          context.lineWidth = line.width;
          Point first = toCanvas(points[0]);
            context..beginPath()
            ..moveTo(first.x, first.y);
          
          for (Point p in points.skip(1)) {
            Point point = toCanvas(p);
            if(PlotWindow.ValidPointForCanvas(p))
              context.lineTo(point.x, point.y);
          }
          context.stroke();
          context.restore();
        }
        if (line.lineType.hasPoints) {
          context.save();
          context.fillStyle = "${line.fillColor}";
          points.map((Point p)=>toCanvas(p)).forEach((Point p) {
            context..moveTo(p.x, p.y)
              ..beginPath()
            ..arc(p.x, p.y, line.pointRadius, 0, 2*math.PI)
              ..fill();
          });
          context.restore();
        }
      }
      if (line.showLinearRegression) {
        line.smoothedPointList.linearRegressionStart = rectangle.left;
        line.smoothedPointList.linearRegressionEnd = rectangle.right;
        var m = line.smoothedPointList.m;
        var c = line.smoothedPointList.c;
        if (m==null || m.isNaN || c==null || c.isNaN ) { continue; }
        Point first = toCanvas(new Point(rectangle.left, m * rectangle.left + c));
        context.save();
        context.setStrokeColorRgb(line.color.red, line.color.green, line.color.blue);
        context.lineWidth = line.width;
        context.setLineDash([5]);
        context..beginPath()
          ..moveTo(first.x, first.y);
        Point last = toCanvas(new Point(rectangle.right, m * rectangle.right + c));
        context.lineTo(last.x, last.y);
        context.stroke();
        context.restore();
      }
    }
    blankOutBorders();
  }

  Iterable<LegendEntry> getLegendEntries() {
    List<LegendEntry> legendEntries = new List<LegendEntry>();
    for(Line line in lines) {
      List<String> sublist = new List<String>()..add(line.name);
      if(line.showLinearRegression)
        sublist.add(line.smoothedPointList.getLinearRegressionString());
      legendEntries.add(new LegendEntry(line.color, sublist, line));
    }
    return legendEntries;
  }

}