part of plot_window;

typedef num NumberFunction(num n);

class AverageMethod {
  String name;
  NumberFunction before;
  NumberFunction after;

  String toString()=>name;
  
  static List<AverageMethod> Cache = new List<AverageMethod>();
  
  static AverageMethod ARITHMETIC = new AverageMethod._intern("Arithmetic",
      (num n)=>n, (num n)=>n);
    
  static AverageMethod GEOMETRIC = new AverageMethod._intern("Geometric",
      (num n)=>math.log(n), (num n)=>math.exp(n));

  bool operator ==(Object other) {
    if(!other is AverageMethod) return false;
    AverageMethod otherAverageMethod = other;
    return this.name==otherAverageMethod.name;
  }

  AverageMethod._intern(this.name, this.before, this.after) {
    Cache.add(this);
  }

  static AverageMethod GetAverageMethod(String s)=>Cache.singleWhere((AverageMethod am)=>am.name==s);
  static bool Exists(String s)=>Cache.where((AverageMethod am)=>am.name==s).isNotEmpty;


  static Init() {
    ARITHMETIC; GEOMETRIC;
  }

}


class PointSmoother implements Performable {
  AverageMethod averageMethod;
  int width;

  String toString()=>"${this.averageMethod.name} (${this.width})";

  Performer get performer {
    SettingsGroup sg = new SettingsGroup("point smoother")
    ..addPerformer(new DataListSetting("method", this.averageMethod, AverageMethod.Cache, (String s) {
        if (AverageMethod.Exists(s))
          this.averageMethod = AverageMethod.GetAverageMethod(s);
      }))
    ..addPerformer(new NumSetting("Width", this.width, (num n) {
        if (n<0) n=0;
        this.width=n~/1;
      }));
    return sg;
  }
  
  bool operator ==(Object other) {
    if(!other is PointSmoother) return false;
    PointSmoother otherPointSmoother = other;
    return this.averageMethod==otherPointSmoother.averageMethod && this.width==otherPointSmoother.width;
  }

  

  PointSmoother(this.averageMethod, this.width);

  List<Point> smooth(Iterable<Point> pointsIterable) {
    List<Point> points = pointsIterable.toList();
    if (points.length < 2 * width + 1) {
      var oldwidth = width;
      width=points.length~/2 - 1;
      print("PointSmoother: points list too short. shortening $oldwidth to $width");
    }

    List<num> input = points.map((Point p)=>averageMethod.before(p.y)/(2*width+1)).toList();
    var vals = new List<num>(points.length - 2*width);
    vals[0] = input.take(2*width +1).fold(0.0, (num total, num elem )=>total+elem);
    for (int i=1; i<vals.length; i++) {
      vals[i] = vals[i-1] - input[i-1] + input[i+2*width];
    }

    var ret = new List<Point>(vals.length);
    for (int i=0; i<vals.length; i++) {
      ret[i] = new Point(points[width+i].x, averageMethod.after(vals[i]));
    }
    return ret;
  }
  
}